from PyQt4 import QtCore, QtGui, uic

from . import __path__
from call import Call

(Ui_CallLogDetail, CallLogDetailBaseClass) = uic.loadUiType(__path__[0]+"/ui/call_log_detail.ui")

class CallLogDetail(CallLogDetailBaseClass, Ui_CallLogDetail):
  def __init__(self, parent):
    CallLogDetailBaseClass.__init__(self, parent)
    self.setupUi(self)
    self.call=None

  def setCall(self, call):
    self.call=call

    if call.type==Call.OUTGOING:
      type="Outgoing call"
    elif call.status==Call.ESTABLISHED:
      type="Incoming call"
    else:
      type="Missed call"
    self.type.setText(type)

    name=call.name
    if not name:
      name=call.phone_number
      if name=="unknown":
        # can't add an unknown contact
        self.add_to_contacts.hide()
      else:
        self.add_to_contacts.show()
    else:
      # if a contact name has been found, so no need to add it to contacts
      self.add_to_contacts.hide()

    self.contact.setText(name)
    self.date_time.setText(QtCore.QDateTime.fromTime_t(call.date_time).toString(QtCore.Qt.LocaleDate))
    d=call.duration/1000
    h=d/3600
    m=(d-h*3600)/60
    s=(d-h*3600-m*60)
    self.duration.setText("%02d:%02d:%02d" % (h, m, s))

class CallLog(QtGui.QStackedWidget):
  class Entry():
    def __init__(self, row):
      for i in ("date_time", "phone_number", "duration", "type", "status", "ack", "name"):
        setattr(self, i, row[i])

  class Model(QtCore.QAbstractListModel):
    def __init__(self, parent, db):
      QtCore.QAbstractListModel.__init__(self, parent)

      self.db=db

      self.loadCalls()

    def loadCalls(self):
      query=self.selectCalls()
      
      self.calls=list()
      for row in query:
        self.calls.append(CallLog.Entry(row))
    
    def selectCalls(self, where="", values=None):
      query=self.db.cursor()
      sql="select date_time, l.phone_number as phone_number, duration, type, status, ack, name from call_log l left join contact c on l.phone_number=c.phone_number "
      if where: sql+=where
      sql+="group by date_time order by date_time desc"

      if values:
        query.execute(sql, values)
      else:
        query.execute(sql)
      return query

    def newCall(self, call):
      # reload the call log entry from the database, so that it is properly formatted as a row
      query=self.selectCalls("where date_time=?", (call.date_time.toTime_t(),))
      self.beginInsertRows(self.index(0, 0), 0, 0)
      self.calls.insert(0, CallLog.Entry(query.fetchone()))
      self.endInsertRows()

    def rowCount(self, parent):
      return len(self.calls)

    def data(self, index, role):
      if role==QtCore.Qt.DisplayRole:
        return QtCore.QVariant(self.calls[index.row()])
      else:
        return QtCore.QVariant()

  class ItemDelegate(QtGui.QAbstractItemDelegate):
    def __init__(self, parent):
      QtGui.QAbstractItemDelegate.__init__(self, parent)

      self.pixmap_incoming=incoming_pixmap=QtGui.QPixmap("widgets/network/phone/icons/prevsection.png")
      self.pixmap_outgoing=QtGui.QPixmap("widgets/network/phone/icons/nextsection.png")
      self.pixmap_missed=QtGui.QPixmap("widgets/network/phone/icons/button_cancel.png")

    def paint(self, painter, option, index):
      call=index.data().toPyObject()

      # find and draw the pixmap according to the call type and status
      # actually only 3 basic cases are taken into account, more sophisticated processing could be done here
      if call.type==Call.OUTGOING:
        p=self.pixmap_outgoing
      elif call.status==Call.ESTABLISHED:
        p=self.pixmap_incoming
      else:
        p=self.pixmap_missed

      if option.state & QtGui.QStyle.State_Selected:
        painter.fillRect(option.rect, option.palette.highlight())
      else:
        # if a missed call has not yet been acknowledged, paint it yellow
        if p is self.pixmap_missed and not call.ack:
          painter.fillRect(option.rect, QtCore.Qt.yellow)
        painter.drawLine(option.rect.bottomLeft(), option.rect.bottomRight())

      # paint the icon for the call type
      painter.drawPixmap(option.rect.topLeft()+QtCore.QPoint(4,4), p)
     
      # draw the contact in big, use the name if available, otherwise use the phone_number
      contact=call.name
      if not contact:
        contact=call.phone_number
      r=QtCore.QRect(option.rect)
      r.setLeft(40)
      r.setWidth(r.width()/2)
      if p is self.pixmap_missed:
        # paint contact name in red for a missed call
        old_color=painter.pen().color()
        painter.setPen(QtCore.Qt.red)
      font=painter.font()
      font.setBold(True)
      old_point_size=font.pointSize()
      font.setPointSize(16)
      painter.drawText(r, QtCore.Qt.AlignHCenter|QtCore.Qt.AlignVCenter, contact)
      font.setPointSize(old_point_size)
      font.setBold(False)
      if p is self.pixmap_missed:
        painter.setPen(old_color)

      # draw the date in little
      date_time=str(QtCore.QDateTime.fromTime_t(call.date_time).toString(QtCore.Qt.LocaleDate))
      r.translate(r.width(), 0)
      painter.drawText(r, QtCore.Qt.AlignHCenter|QtCore.Qt.AlignVCenter, date_time)

    def sizeHint(self, option, index):
      # we assume a 32 pixel icon height, and 4 pixels top/bottom margin
      return QtCore.QSize(0, 40)

  def __init__(self, parent, db):
    QtGui.QStackedWidget.__init__(self, parent)

    self.db=db

    self.list=QtGui.QListView(self)
    self.model=CallLog.Model(self, db)
    self.item_delegate=CallLog.ItemDelegate(self)
    self.list.setModel(self.model)
    self.list.setItemDelegate(self.item_delegate)
    self.connect(self.list, QtCore.SIGNAL("clicked(const QModelIndex &)"), self.onCallClicked)

    self.call_log_detail=CallLogDetail(self)
    self.connect(self.call_log_detail.add_to_contacts, QtCore.SIGNAL("clicked()"), self.onAddToContactsClicked)
    self.connect(self.call_log_detail.back_to_call_log, QtCore.SIGNAL("clicked()"), self.onBackToCallLogClicked)

    self.addWidget(self.list)
    self.addWidget(self.call_log_detail)
    self.setCurrentWidget(self.list)

  def onCallClicked(self, index):
    call=index.data().toPyObject()

    # acknowledge the call as soon as it is clicked
    if not call.ack:
      self.db.cursor().execute("update call_log set ack=1 where date_time="+str(call.date_time))
      call.ack=1

    self.call_log_detail.setCall(call)

    self.setCurrentWidget(self.call_log_detail)

  def onAddToContactsClicked(self):
    self.emit(QtCore.SIGNAL("createContact"), self.call_log_detail.call.phone_number)
    self.setCurrentWidget(self.list)

  def onBackToCallLogClicked(self):
    self.setCurrentWidget(self.list)

  def logCall(self, call):
    """Insert a new call into the log."""
    c=self.db.cursor()
    if call.duration:
      duration=call.duration.elapsed()
    else:
      duration=0
    values=(call.date_time.toTime_t(), call.phone_number, duration, call.type, call.status, 0)
    c.execute("insert into call_log(date_time, phone_number, duration, type, status, ack) values(?, ?, ?, ?, ?, ?)", values)

    self.model.newCall(call)

  def reload(self):
    self.model.loadCalls()
    self.model.reset()

