import dbus
from PyQt4 import QtCore, QtGui, uic
import sqlite3

from . import __path__
from core.auimd_widget import *

from startup import Startup
from call_ui import CallUi
from dialer import Dialer
from pin_dialog import PinDialog
from call import Call
from call_log import CallLog
from contacts import Contacts


class Phone(QtGui.QStackedWidget, AuimdWidget):
  @staticmethod
  def defaultWidgetIcon():
    return QtGui.QIcon(__path__[0]+"/icons/voicecall.png")

  @staticmethod
  def defaultWidgetTitle():
    return "Phone"

  @staticmethod
  def getCategory():
    return "Network"

  @staticmethod
  def monoInstance():
    return True

  def __init__(self, parent, *args):
    QtGui.QStackedWidget.__init__(self, parent)
    AuimdWidget.__init__(self, parent)

    # dict of id -> Call and id -> CallUi
    self.calls={}
    self.call_uis={}

    # init UI components
    self.startup=Startup(self)
    self.main_ui=QtGui.QTabWidget(self)
    self.addWidget(self.startup)
    self.addWidget(self.main_ui)
    self.setCurrentWidget(self.startup)

    # open the database for call log, contacts, SMSes, etc.
    self.initDatabase()

    # continue with UI init, now that the database is ready (needed for contacts, call log)
    self.dialer=Dialer(self.main_ui)
    self.connect(self.dialer, QtCore.SIGNAL("callNumber"), self.initiateOutgoingCall)
    self.main_ui.addTab(self.dialer, "Dialer")

    self.call_log=CallLog(self.main_ui, self.db)
    self.connect(self.call_log, QtCore.SIGNAL("createContact"), self.onCreateContact)
    self.main_ui.addTab(self.call_log, "Call log")

    self.contacts=Contacts(self.main_ui, self.db, self)
    self.connect(self.contacts, QtCore.SIGNAL("createContact"), self.onCreateContact)
    self.connect(self.contacts, QtCore.SIGNAL("callContact"), self.initiateOutgoingCall)
    self.connect(self.contacts, QtCore.SIGNAL("contactModified"), self.onContactModified)
    self.main_ui.addTab(self.contacts, "Contacts")

    self.right_button.setText("Close")

    # init DBus components
    try:
      self.system_bus=dbus.SystemBus()
      
      self.usage=self.system_bus.get_object("org.freesmartphone.ousaged", "/org/freesmartphone/Usage")
      self.usage_iface=dbus.Interface(self.usage, "org.freesmartphone.GSM.Device")

      self.gsm_device=self.system_bus.get_object("org.freesmartphone.ogsmd", "/org/freesmartphone/GSM/Device")
      self.gsm_device_iface=dbus.Interface(self.gsm_device, "org.freesmartphone.GSM.Device")
      self.gsm_network_iface=dbus.Interface(self.gsm_device, "org.freesmartphone.GSM.Network")
      self.gsm_call_iface=dbus.Interface(self.gsm_device, "org.freesmartphone.GSM.Call")
      self.gsm_sim_iface=dbus.Interface(self.gsm_device, "org.freesmartphone.GSM.SIM")

      self.gsm_call_iface.connect_to_signal("CallStatus", self.onCallStatus)
 
      self.init=self.initiator()
      self.init.next()
    except:
      print "dbus init failed"
      # FIXME temporary test purpose
      self.setCurrentWidget(self.main_ui)

    # FIXME simulation
    self.simu_state=0

  # FIXME maybe need a finer granularity
  #def keyboardAllowed(self):
  #  return False

  def initDatabase(self):
    # open the database where contacts, call logs and SMSes are stored
    # (in theory contacts should not be dependent from the phone, but lets keep things simple for now)
    # in order to keep things located on the same place, reuse the settings location...
    path=QtCore.QSettings("auimd", "Phone").fileName()+".sqlite"
    self.db=sqlite3.connect(str(path), isolation_level=None)
    self.db.row_factory=sqlite3.Row

    # check whether this database has already been initialized
    c=self.db.cursor()
    try:
      c.execute("select * from call_log")
    except:
      c.execute("create table call_log(date_time primary key, phone_number, duration, type, status, ack)")
      c.execute("create table contact(id integer primary key, name, phone_number)")
    c.close()

  def initiator(self):
    # first request the resource
    self.displayInitMessage("Initializing GSM resources")
    self.usage.RequestResource("GSM", reply_handler=self.onRequestResourceReply, error_handler=self.onRequestResourceError)
    yield

    # antenna power up
    self.displayInitMessage("Powering up antenna")
    self.gsm_device_iface.SetAntennaPower(1, reply_handler=self.onSetAntennaPowerReply, error_handler=self.onSetAntennaPowerError)
    yield

    if not self.antenna_ok:
      # check whether a PIN is needed
      self.displayInitMessage("Does the SIM require a PIN code")
      self.gsm_sim_iface.GetAuthStatus(reply_handler=self.onGetAuthStatusReply, error_handler=self.onGetAuthStatusError)
      yield

      # request the user to enter the PIN code if needed
      if self.auth_status=="SIM PIN":
        self.displayInitMessage("Requesting PIN code")
        pin_dialog=PinDialog()
        res=self.desktop.dialog(self, pin_dialog, "PIN code")

        if res==QtGui.QDialog.Accepted and pin_dialog.pin_code:
          # if a pin code has been entered, check it
          self.displayInitMessage("Validating PIN code")
          self.gsm_sim_iface.SendAuthCode(str(pin_dialog.pin_code), reply_handler=self.onSendAuthCodeReply, error_handler=self.onSendAuthCodeError)
          yield
          if self.auth_status!="READY":
            self.initStepFailed("Wrong PIN code")
        else:
          self.initStepFailed("No PIN given")

    # register to home network
    self.displayInitMessage("Registering to the network")
    self.gsm_network_iface.Register(reply_handler=self.onRegisterReply, error_handler=self.onRegisterError, timeout=300)
    yield

  def displayInitMessage(self, msg):
    self.startup.displayInitMessage(msg)

  def initStepSuccessful(self, msg):
    self.startup.initStepSuccessful(msg)
    try:
      self.init.next()
    except StopIteration:
      self.setCurrentWidget(self.main_ui)
    

  def initStepFailed(self, msg):
    self.startup.initStepFailed(msg)
    del self.init

  def onRequestResourceReply(self):
    self.initStepSuccessful("ok")

  def onRequestResourceError(self, e):
    self.initStepFailed("failed")

  def onSetAntennaPowerReply(self):
    self.antenna_ok=True
    self.initStepSuccessful("ok")

  def onSetAntennaPowerError(self, e):
    self.antenna_ok=False
    self.initStepSuccessful("checking auth status")

  def onGetAuthStatusReply(self, auth_status):
    self.auth_status=auth_status
    self.initStepSuccessful(auth_status)

  def onGetAuthStatusError(self, e):
    self.initStepFailed("error")

  def onSendAuthCodeReply(self):
    self.auth_status="READY"
    self.initStepSuccessful("accepted")
    
  def onSendAuthCodeError(self, e):
    self.initStepFailed("rejected")

  def onRegisterReply(self):
    self.initStepSuccessful("ok")
 
  def onRegisterError(self, e):
    self.initStepFailed("failed")

  def rightButtonAction(self):
    self.desktop.closeWidget(self)


  def minimumSizeHint(self):
      return QtCore.QSize(0, 0)


  def onCallStatus(self, id, status, properties):
    if status=="incoming":
      if id not in self.calls:
        try:
          phone_number=properties["peer"]
        except:
          phone_number="unknown"

        # FIXME find the name of the contact
        call=Call(id, Call.INCOMING, phone_number, "")
        call_ui=self.desktop.createWidget(CallUi, call)
        self.calls[id]=call
        self.call_uis[id]=call_ui
        self.connect(call_ui, QtCore.SIGNAL("callAccepted"), self.onCallAccepted)
        self.connect(call_ui, QtCore.SIGNAL("callEnded"), self.onCallEnded)
        self.connect(call_ui, QtCore.SIGNAL("callRejected"), self.onCallRejected)
        self.connect(call_ui, QtCore.SIGNAL("dtmf"), self.onDtmf)
        self.desktop.showWidget(call_ui)

    elif status=="outgoing":
      call_ui=self.desktop.createWidget(CallUi, self.calls[id])
      self.call_uis[id]=call_ui
      self.connect(call_ui, QtCore.SIGNAL("callCanceled"), self.onCallCanceled)
      self.connect(call_ui, QtCore.SIGNAL("callEnded"), self.onCallEnded)
      self.connect(call_ui, QtCore.SIGNAL("dtmf"), self.onDtmf)
      self.desktop.showWidget(call_ui)

    elif status=="active":
      call=self.calls[id]
      call.status=Call.ESTABLISHED
      call.duration=QtCore.QTime()
      call.duration.start()
      call_ui=self.call_uis[id]
      call_ui.setStatus(status)

    elif status=="release":
      call=self.calls[id]
      
      self.call_log.logCall(call)

      self.desktop.closeWidget(self.call_uis[id])
      del self.calls[id]
      del self.call_uis[id]

  def initiateOutgoingCall(self, phone_number, name=None):
    id=self.gsm_call_iface.Initiate(str(phone_number), "voice")
    call=Call(id, Call.OUTGOING, phone_number, name)
    self.calls[id]=call

  def onCreateContact(self, phone_number=""):
    self.contacts.createContact(phone_number)
    self.main_ui.setCurrentWidget(self.contacts)

  def onCallAccepted(self, call):
    self.gsm_call_iface.Activate(call.id)

  def onCallRejected(self, call):
    call.status=Call.REJECTED
    self.gsm_call_iface.Release(call.id)

  def onCallCanceled(self, call):
    call.status=Call.CANCELED
    self.gsm_call_iface.Release(call.id)

  def onCallEnded(self, call):
    try:
      self.gsm_call_iface.Release(call.id)
    except:
      # FIXME investigate whether it is wrong or not
      print "trying to release an incoming but not yet established call ?"

  def onDtmf(self, tone):
    self.gsm_call_iface.SendDtmf(tone)
  
  def readyToClose(self):
    # avoid closing while one or more calls are in progress
    if len(self.calls)==0:
      #d=QtGui.QMessageBox(QtGui.QMessageBox.Question, "Close the phone", 
      #    "Do you really want to close\nthe phone ?\n(you won't be able to\nreceive calls)", 
      #    QtGui.QMessageBox.Ok|QtGui.QMessageBox.Cancel, self.desktop)
      #return self.desktop.dialog(self, d)==QtGui.QMessageBox.Ok
      return True
    else:
      return False

  def onContactModified(self):
    self.call_log.reload()

  def closeEvent(self, e):
    self.db.close()
    self.usage.ReleaseResource("GSM")

  def logCall(self, call):
    c=self.db.cursor()
    values=(call.date_time.toTime_t(), call.phone_number, call.duration.elapsed(), call.type, call.status, 0)
    c.execute("insert into call_log(date_time, phone_number, duration, type, status, ack) values(?, ?, ?, ?, ?, ?)", values)
    c.close()

  def leftButtonAction(self):
    # FIXME temporary simulation code
    phone_number="1234567891"
    id=1
    if 1:
      if self.simu_state==0:
        self.onCallStatus(id, "incoming", {"peer":phone_number})
        self.onCallStatus(id, "active", {})
        self.simu_state+=1
      elif self.simu_state==1:
        self.onCallStatus(id, "release", {})
        self.simu_state=0
    else:
      if self.simu_state==0:
        call=Call(id, Call.OUTGOING, phone_number, "toto")
        self.calls[id]=call
        self.onCallStatus(id, "outgoing", {"peer":phone_number})
        self.simu_state+=1
      elif self.simu_state==1:
        self.onCallStatus(id, "active", {})
        self.simu_state+=1
      elif self.simu_state==2:
        self.onCallStatus(id, "release", {})
        self.simu_state=0
